import { default as noble } from '@stoprocent/noble';
import randomMac from 'random-mac';
import { spawnSync } from 'node:child_process';
import { Client } from '@elastic/elasticsearch';
const machineId = process.env.MACHINEID || 'Pi4',
    indexBle = process.env.ELASTICINDEX || 'ble',
    seconds: any = process.env.SECONDS || 60,
    counterChangeMac: any = process.env.COUNTERCHANGEMAC || 90,
    waitStartScan: any = process.env.WAITSTARTSCAN || 3

let peers = [],
    counter = 0

const changeMacBle = () => {
    const newMac = randomMac('00:12:34').split(':').reverse().map(data => '0x' + data)
    spawnSync('hcitool', ['-i', 'hci0', 'cmd', '0x3f', '0x001'].concat(newMac))
    spawnSync('hciconfig', ['hci0', 'reset'])
},
    transformObject = item => {
        if (item && typeof item === 'object') {
            Object.entries(item).map(([key, value]) => {
                if (value && Buffer.isBuffer(value)) {
                    item[key] = Buffer.from(value).toString('hex')
                } else {
                    item[key] = transformObject(value)
                }
            })
        }
        return item
    },
    mainBle = (client: Client) => {
        noble.on('discover', peripheral => {
            if (peripheral && peripheral.id && peripheral.uuid) {
                const peer = transformObject({
                    id: peripheral.id,
                    uuid: peripheral.uuid,
                    address: peripheral.address,
                    addressType: peripheral.addressType,
                    connectable: peripheral.connectable,
                    advertisement: peripheral.advertisement,
                    rssi: peripheral.rssi,
                    services: peripheral.services,
                    mtu: peripheral.mtu,
                    state: peripheral.state,
                    seen: new Date(),
                    machine: machineId
                })
                peers.push(peer)
            }
        })
        noble.on('scanStart', () => {
            peers = []
            setTimeout(() => {
                noble.stopScanning()
            }, seconds * 1000)
        })
        noble.on('scanStop', async () => {
            if (counter >= counterChangeMac) {
                changeMacBle()
                counter = 0
            } else {
                counter++
            }
            const conn = await client.ping()
            if (conn) {
                client.helpers.bulk({
                    datasource: peers,
                    onDocument(doc) {
                        return {
                            index: { _index: indexBle }
                        }
                    }
                })
            } else {
                throw new Error('disconnected')
            }
            setTimeout(() => {
                noble.startScanning([], false, err => {
                    if (err) {
                        throw err
                    }
                })
            }, waitStartScan * 1000)
        })
        noble.on('stateChange', state => {
            if (state === 'poweredOn') {
                noble.startScanning([], false, err => {
                    if (err) {
                        throw err
                    }
                })
            }
        })
    }

(async () => {
    try {
        const client = new Client({ node: process.env.ELASTICSEARCHNODE || 'http://elasticsearch-node:9200' })
        if (!(await client.indices.exists({ index: indexBle }))) {
            await client.indices.create({
                index: indexBle,
                mappings: require('./ble-index-mapping.json')
            })
        }
        changeMacBle()
        mainBle(client)
    } catch (e) {
        console.error(e)
        process.exit(1)
    }
})()
